import string
import commands
import sys
    
import os
import os.path

from droptarget import *
from comments import *
from CommentDialog import *

from uctree import *

import wx

MULTIVALENT_INFO = None
cp = os.environ.get("CLASSPATH", "") + \
     ":/usr/local/pkgs/Multivalent" + \
     ":/home/titus/bin/"

for c in cp.split(":"):
    if os.access(c + "/" + "Multivalent.jar", os.R_OK):
        MULTIVALENT_INFO = "java -classpath %s/Multivalent.jar tool.pdf.Info"

if MULTIVALENT_INFO is None:
    sys.exit("Cannot find Multivalent.jar in $CLASSPATH:\n" +
             "Please download and install Multivalent or set CLASSPATH")
        
SCROLL_SIZE = 10

class AnnotateFrame(wx.Frame):
    def __init__(self, parent, *args, **kwds):
        wx.Frame.__init__(self, parent, *args, **kwds)
        self.p = p = wx.Panel(self, -1)
        p.SetAutoLayout(True)

        self.parent = parent
        
        self.commentTree = UserCommentTree(self.parent, self.parent.a, p, -1,
                                           style=wx.TR_HAS_BUTTONS|\
                                           wx.TR_LINES_AT_ROOT|\
                                           wx.TR_DEFAULT_STYLE|\
                                           wx.SUNKEN_BORDER)

        self.addCommentBtn = wx.Button(p, -1, "Add Comment")
        
        self.commentText = wx.StaticText(p, -1, "Comment Info")
        self.bitmapPanel = wx.ScrolledWindow(p, -1)
        self.bitmapPanel.SetScrollbars(SCROLL_SIZE, SCROLL_SIZE, 60, 80)
        self.bitmapPanel.SetSize((600, 800))
        
        self.pointsText = wx.StaticText(p, -1, "Points  ")
        self.newCommentBtn = wx.Button(p, -1, "New Comment")
        self.prevPageBtn = wx.Button(p, -1, "Prev Page")
        self.prevSubBtn = wx.Button(p, -1, "Prev Sub")
        self.nextSubBtn = wx.Button(p, -1, "Next Sub")
        self.nextPageBtn = wx.Button(p, -1, "Next Page")
        self.repeatBtn = wx.Button(p, -1, "Repeat Last")        
        self.repeatBtn.Enable(False)
        self.doneBtn = wx.Button(p, -1, "Done")
        self.rubricItem = wx.StaticText(p, -1, "label_32")
        self.rubricItem.SetSize((200,20))
        self.rubricItem.SetMinSize((200,20))
        self.pageLabel = wx.StaticText(p, -1, "Current Page")
        self.pageTextCtrl = wx.TextCtrl(p, -1, "")
        self.subLabel = wx.StaticText(p, -1, "Current Submission")
        self.subTextCtrl = wx.TextCtrl(p, -1, "", style=wx.TE_READONLY)

        self.blankPanel = wx.Panel(p, -1)

        self.__set_properties()
        self.__do_layout()
        
        self.curpage = 1
    
        username = self.parent.subTree.getUsername()

        while self.parent.a.submissions[username].pdf == None:
            try:
                self.onNextSub(0)
            except:
                err = wx.MessageDialog(self,
                                      "No viewable files!\n" +
                                      "This is something that shouldn't\n"+ \
                                      "have happened.  Very important\n" +\
                                      "for you to report this.\n",
                                      "Error!",
                                      style=wx.ICON_ERROR|wx.OK)
                err.ShowModal()
                err.Destroy()
                self.onDone()

        username = self.username = self.parent.subTree.getUsername()
            
        self.SetTitle("Annotate: " + username)

        wx.EVT_BUTTON(self.addCommentBtn, -1, self.onAdd)

        wx.EVT_BUTTON(self.repeatBtn, -1, self.onRepeat)

        wx.EVT_BUTTON(self.prevPageBtn, -1, self.onPrevPage)
        wx.EVT_BUTTON(self.prevSubBtn, -1, self.onPrevSub)
        wx.EVT_BUTTON(self.nextSubBtn, -1, self.onNextSub)
        wx.EVT_BUTTON(self.nextPageBtn, -1, self.onNextPage)

        wx.EVT_BUTTON(self.newCommentBtn, -1, self.onNewComment)

        wx.EVT_BUTTON(self.doneBtn, -1, self.onDone)
        wx.EVT_CLOSE(self, self.onDone)

        wx.EVT_TREE_SEL_CHANGED(self.commentTree, -1, self.changeSelComment)

        self.usernames = self.parent.a.submissions.keys()
        self.usernames.sort()

        wx.EVT_KEY_DOWN(self.bitmapPanel, self.onKeyDown)
        wx.EVT_KEY_DOWN(self.commentTree, self.onKeyDown)
        wx.EVT_KEY_DOWN(self.p, self.onKeyDown)

        wx.EVT_LEFT_DOWN(self.bitmapPanel, self.onMouseDown)
        wx.EVT_MOTION(self.bitmapPanel, self.onMouseMove)
        wx.EVT_LEFT_DOWN(self, self.onElseClick)
        
        wx.EVT_PAINT(self.bitmapPanel, self.onPaint)

        wx.EVT_SIZE(self, self.onSize)

        self.dt = AgarDropTarget(self.getDropData)
        self.data = wx.CustomDataObject(wx.CustomDataFormat("AgarComment"))
        self.dt.SetDataObject(self.data)
        self.SetDropTarget(self.dt)


        self.greenMark = wx.Bitmap(self.parent.a.agarPath + 
                                  "/graphics/greenex.png")

        # wx.Python 2.4 / 2.5 compatibility
        try:
            mask = wx.Mask
        except:
            mask = wx.MaskColour
            
        self.greenMark.SetMask(mask(self.greenMark,
                                            wx.Colour(255,255,255)))
        self.greenMarkH = wx.Bitmap(self.parent.a.agarPath + \
                                  "/graphics/greenexHighlight.png")
        self.greenMarkH.SetMask(mask(self.greenMarkH,
                                            wx.Colour(255,255,255)))
        
        self.redMark = wx.Bitmap(self.parent.a.agarPath + \
                                  "/graphics/redex.png")
        self.redMark.SetMask(mask(self.redMark,
                                            wx.Colour(255,255,255)))
        self.redMarkH = wx.Bitmap(self.parent.a.agarPath + \
                                  "/graphics/redexHighlight.png")
        self.redMarkH.SetMask(mask(self.redMarkH,
                                            wx.Colour(255,255,255)))

        self.excMark = wx.Bitmap(self.parent.a.agarPath + \
                                  "/graphics/exc.png")
        self.excMark.SetMask(mask(self.excMark,
                                            wx.Colour(255,255,255)))
        self.excMarkH = wx.Bitmap(self.parent.a.agarPath + \
                                  "/graphics/excHighlight.png")
        self.excMarkH.SetMask(mask(self.excMarkH,
                                            wx.Colour(255,255,255)))

        self.targetMode = False
        self.comment = None
        self.curComment = None
        self.selSkip = False

        self.changeSelComment(0)

        self.updateSub()
        self.updatePage()
        self.redraw()
        
    def __set_properties(self):
        self.SetTitle("Annotate")
        self.SetSize((850, 875))
        self.commentText.SetSize((180, 150))
        self.commentText.SetMinSize((180, 150))

    def __do_layout(self):
        # begin wx.Glade: AnnotateFrame.__do_layout
        self.sizer = sizer = wx.BoxSizer(wx.VERTICAL)
        topSizer = wx.FlexGridSizer(1, 2, 0, 0)
        botSizer = wx.FlexGridSizer(1, 2, 0, 0)            
        
        commentSizer = wx.BoxSizer(wx.VERTICAL)
        commentInfoSizer = wx.FlexGridSizer(2, 2, 0, 0)
        controlSizer = wx.BoxSizer(wx.VERTICAL)
        buttonsSizer = wx.BoxSizer(wx.HORIZONTAL)
        infoSizer = wx.BoxSizer(wx.HORIZONTAL)

        # This is the static box that goes around the information for
        # self.commentTExt
        commentBox = wx.StaticBoxSizer(wx.StaticBox(self.p, -1, "Comment"),
                                      wx.HORIZONTAL)
        commentBox.Add(self.commentText)
        
        commentSizer.Add(self.commentTree, 1, wx.EXPAND)
        commentSizer.Add(self.addCommentBtn, 0, wx.EXPAND)
        commentSizer.Add(self.repeatBtn, 0, wx.EXPAND)
        commentSizer.Add(commentBox, 0, wx.EXPAND)

        commentInfoSizer.Add(self.pointsText, 1, wx.EXPAND)
        commentInfoSizer.Add(self.newCommentBtn, 1, wx.EXPAND)
        commentInfoSizer.Add(self.rubricItem, 1, wx.EXPAND)
        commentInfoSizer.Add(self.blankPanel, 1, wx.EXPAND)

        buttonsSizer.Add(self.prevPageBtn, 1, wx.EXPAND)
        buttonsSizer.Add(self.prevSubBtn, 1, wx.EXPAND)
        buttonsSizer.Add(self.nextSubBtn, 1, wx.EXPAND)
        buttonsSizer.Add(self.nextPageBtn, 1, wx.EXPAND)
        buttonsSizer.Add(self.doneBtn, 1, wx.EXPAND)

        infoSizer.Add(self.pageLabel, 1, wx.EXPAND)
        infoSizer.Add(self.pageTextCtrl, 1, wx.EXPAND)
        infoSizer.Add(self.subLabel, 1, wx.EXPAND)
        infoSizer.Add(self.subTextCtrl, 1, wx.EXPAND)
        
        controlSizer.Add(buttonsSizer, 1, wx.EXPAND)
        controlSizer.Add(infoSizer, 1, wx.EXPAND)

        topSizer.Add(commentSizer, 1, wx.EXPAND)
        topSizer.Add(self.bitmapPanel, 1, wx.EXPAND|wx.ADJUST_MINSIZE|wx.FIXED_MINSIZE)
        botSizer.Add(commentInfoSizer, 0, wx.EXPAND)
        botSizer.Add(controlSizer, 0, wx.EXPAND)
        sizer.Add(topSizer, 1, wx.EXPAND)
        sizer.Add(botSizer, 0, wx.EXPAND)
        
        self.p.SetSizer(self.sizer)
        self.SetAutoLayout(True)
        self.bitmapPanel.SetSize((600, 800))
        self.sizer.Layout()
        self.sizer.Fit(self)

    def getDropData(self, x, y, data):
        if self.dt.GetData():
            commentName = str(self.data.GetData())

            if commentName not in self.parent.a.userComments:
                return

            try:
                name = self.parent.subtree.getUsername()
                sub = self.parent.a.submissions[name]
            except:
                return

            sub.comments.addComment(self.a.userComments[commentName])

            self.parent.subtree.update()
            self.parent.resultTree.update()

    def onSize(self, event):
        if not hasattr(self, "sizes"):
            self.sizes = (800, 875)
            
        self.p.SetSize(event.GetSize())
        x = event.GetSize()[0] - (self.sizes[0] - 600)
        y = event.GetSize()[1] - (self.sizes[1] - 800)
        self.bitmapPanel.SetSize((x, y))
        self.bitmapPanel.SetMinSize((x, y))
        self.sizer.RecalcSizes()
        self.sizer.Layout()

    def onAdd(self, event):
        selId = self.commentTree.GetSelection()
        if not selId.IsOk():
            return
        
        commentName = str(self.commentTree.GetItemText(selId))
        if commentName == "Comments" or \
               commentName not in self.parent.a.userComments:
            return
        
        comment = self.parent.a.userComments[commentName]

        self.comment = comment
        self.bitmapPanel.SetCursor(wx.CROSS_CURSOR)
        self.targetMode = True
        
    def onKeyDown(self, event):
        if event.GetKeyCode() == 127 and \
               event.GetEventObject() == self.bitmapPanel:
            if self.curComment != None:
                sub = self.parent.a.submissions[self.username]
                ind = sub.locations.index(self.curComment)
                del sub.locations[ind]

                ind = sub.comments.index(self.curComment[0])
                del sub.comments[ind]

                self.curComment = None
                self.comment = None

                self.redraw()
        elif event.GetKeyCode() == 317:
            if self.username != self.usernames[0]:
                self.onPrevSub(None)
        elif event.GetKeyCode() == 319:
            if self.username != self.usernames[-1]:
                self.onNextSub(None)
        elif event.GetKeyCode() == 316:
            if self.curpage != 1:
                self.onPrevPage(None)            
        elif event.GetKeyCode() == 318:
            if self.curpage != self.pages:
                self.onNextPage(None)
        elif event.GetKeyCode() == 78:
            self.onNewComment(None)
        elif event.GetKeyCode() == 27:
            self.onDone(None)
        elif event.GetKeyCode() == 65:
            self.onAdd(None)
        elif event.GetKeyCode() == 82:
            self.onRepeat(None)
                    
    def onElseClick(self, event):
        if self.targetMode:
            self.targetMode = False
            self.bitmapPanel.SetCursor(wx.STANDARD_CURSOR)

    def onPaint(self, event):
        dc = wx.PaintDC(self.bitmapPanel)
        self.bitmapPanel.PrepareDC(dc)
        dc.Clear()
        self.draw(dc)

    def draw(self, dc):
        if not self.bitmap:
            dc.EndDrawing()
            return

        x = int((600 - self.bitmap.GetWidth()) / 2)
        y = int((800 - self.bitmap.GetHeight()) / 2)

        dx, dy = self.bitmapPanel.GetViewStart()

        dc.SetBrush(wx.TRANSPARENT_BRUSH)
        dc.SetPen(wx.BLACK_PEN)
        dc.BeginDrawing()
        dc.DrawBitmap(self.bitmap, x, y, False)

        sub = self.parent.a.submissions[self.username]
        for comment, (cx, cy, page) in sub.comments.getCommentLocs():
            if page != self.curpage:
                continue            

            origcx, origcy = cx, cy

            if comment.value < 0 or (comment.value == 0 and comment.op == '='):
                if [comment, (origcx, origcy, page)] != self.curComment:
                    dc.DrawBitmap(self.redMark, cx, cy, True)
                else:
                    dc.DrawBitmap(self.redMarkH, cx, cy, True)
                    dc.DrawRectangle(cx, cy,
                                     self.redMarkH.GetWidth(),
                                     self.redMarkH.GetHeight())

            elif comment.value == 0:
                if [comment, (origcx, origcy, page)] != self.curComment:
                    dc.DrawBitmap(self.excMark, cx, cy, True)
                else:
                    dc.DrawBitmap(self.excMarkH, cx, cy, True)
                    dc.DrawRectangle(cx, cy,
                                     self.excMarkH.GetWidth(),
                                     self.excMarkH.GetHeight())
            else:                
                if [comment, (origcx, origcy, page)] != self.curComment:
                    dc.DrawBitmap(self.greenMark, cx, cy, True)
                else:
                    dc.DrawBitmap(self.greenMarkH, cx, cy, True)
                    dc.DrawRectangle(cx, cy,
                                     self.greenMarkH.GetWidth(),
                                     self.greenMarkH.GetHeight())

        dc.EndDrawing()

    def commentDrag(self, event):
        if not hasattr(self.parent, "dragWarning"):
            self.parent.dragWarning = 1
            dlg = wx.MessageDialog(self,
                                  "Drag and drop has been disabled until "+\
                                  "the underlying libraries solve the\n" +\
                                  "segfault contained there.\n\n" + \
                                  "Use the 'Add' button, below.",
                                  "Feature disabled.",
                                  style=wx.ICON_INFORMATION|wx.OK)
            dlg.ShowModal()
            dlg.Destroy()
            return
        return
        
        selId = self.commentTree.GetSelection()
        if not selId.IsOk():
            return
        commentName = str(self.commentTree.GetItemText(selId))
        
        cdata = wx.CustomDataObject(wx.CustomDataFormat("Comment"))
        cdata.SetData(str(commentName))
        ds = wx.DropSource(self.commentTree)
        ds.SetData(cdata)
        ds.DoDragDrop()
        
    def changeSelComment(self, event):
        if self.selSkip:
            self.selSkip = False
            return
        
        myTree = self.commentTree
        curId = self.commentTree.GetSelection()
        if not curId.IsOk():
            return

        name = str(myTree.GetItemText(curId))
        if name == "Comments":
            name = ""

        try:
            self.comment = self.parent.a.userComments[name]
        except KeyError:
            self.comment = None        

        if self.comment is None:
            self.commentText.SetLabel("No Comment Selected")
            self.pointsText.SetLabel("Points")
            self.rubricItem.SetLabel("Rubric Item")
        else:
            self.commentText.SetLabel(self.comment.comment)
            self.pointsText.SetLabel("Points: %s %s" % (self.comment.op,
                                                        self.comment.value))

            self.rubricItem.SetLabel("Rubric Item: " + \
                                     self.comment.test.name)        
    
    def onMouseDown(self, event):
        (x, y) = event.GetPosition()

        width = self.bitmap.GetWidth()
        height = self.bitmap.GetHeight()
        
        dx = (600 - width) / 2
        dy = (800 - height) / 2
        
        if self.targetMode:
            x += dx + int(self.redMark.GetWidth() * 1.25)
            y += int(dy + self.redMark.GetHeight() / 2)

            xpos, ypos = self.bitmapPanel.GetViewStart()
            x += xpos * SCROLL_SIZE
            y += ypos * SCROLL_SIZE            
            
            self.targetMode = False
            self.dropped(x, y, self.comment)
            self.bitmapPanel.SetCursor(wx.STANDARD_CURSOR)
            self.parent.subTree.update()
            self.parent.resultTree.update()
            return

        try:
            sub = self.parent.a.submissions[self.username]
        except KeyError:
            return

        ddx, ddy = self.bitmapPanel.GetViewStart()        
        x += dx + ddx * SCROLL_SIZE
        y += dy + ddy * SCROLL_SIZE
        
        poss = {}
        for ind, (comment, (cx, cy, page)) in enumerate(sub.comments.getCommentLocs()):
            if page != self.curpage:
                continue

            if abs(cx - x) < 20 and abs(cy - y) < 20:
                poss[abs(cx - x) + abs(cy - y)] = ind
                        
        if not poss:
            self.curComment = None
            self.redraw()            
            return
        
        best = min(poss.keys())

        self.curComment = sub.comments.getCommentLocs()[poss[best]]
        self.comment = self.curComment[0]

        for cname in self.parent.a.userComments:
            c = self.parent.a.userComments[cname]

            if c is self.comment:
                self.commentTree.selectedPath = [cname]
                self.commentTree.update()
                break
        
        self.redraw()

    def onMouseMove(self, event):
        if self.curComment == None or not event.Dragging():
            return
        
        (x, y) = event.GetPosition()

        width = self.bitmap.GetWidth()
        height = self.bitmap.GetHeight()
        
        dx = (600 - width) / 2
        dy = (800 - height) / 2
        
        try:
            sub = self.parent.a.submissions[self.username]
            ind = sub.comments.index(self.curComment[0])
        except:
            return

        ddx, ddy = self.bitmapPanel.GetViewStart()        
        x += dx + ddx * SCROLL_SIZE + int(self.redMark.GetWidth() / 2)
        y += dy + ddy * SCROLL_SIZE + int(self.redMark.GetHeight() / 2)

        sub.comments.locations[ind] = (x, y, self.curpage)
        self.curComment = (sub.comments[ind], sub.comments.locations[ind])
        self.comment = sub.comments[ind]
        self.redraw()
        

    def onRepeat(self, event):
        if not hasattr(self, "repeat"):
            return

        (x, y, comment) = self.repeat

        self.dropped(x, y, comment)

    def dropped(self, x, y, comment):
        self.repeat = (x, y, comment)
        self.repeatBtn.Enable(True)

        x -= int(self.redMark.GetWidth() / 2)
        y -= int(self.redMark.GetHeight() / 2)

        sub = self.parent.a.submissions[self.username]
        if comment in sub.comments:
            dlg = wx.MessageDialog(self,
                                  "Add Duplicate Comment?",
                                  "Confirm",
                                  style=wx.ICON_QUESTION|wx.YES_NO)
            response = dlg.ShowModal()
            dlg.Destroy()
            if response != wx.ID_YES:
                return

        
        sub.comments.addCommentLoc(comment,
                                   x, y,
                                   self.curpage)

        self.redraw()

    def redraw(self):
        dc = wx.ClientDC(self.bitmapPanel)
        self.bitmapPanel.PrepareDC(dc)
        dc.Clear()
        self.draw(dc)               
        
    def updatePage(self):
        self.pageTextCtrl.SetValue(str(self.curpage))
        self.checkBounds()

        self.bitmap = wx.Bitmap(self.makeFilename(self.username, self.curpage))
        self.redraw()

    def updateSub(self):
        # Are we on the first/last page of this submission?
        if self.parent.a.submissions[self.username].pdf == None:
            dlg = wx.MessageDialog(self,
                                  "PDF file missing or unreadable:\n" +
                                  self.parent.a.submissions[self.username].pdf,
                                  "Error",
                                  style=wx.ICON_ERROR|wx.OK)
            dlg.ShowModal()
            dlg.Destroy()

            self.bitmap = None
            self.pages = 1
            self.curpage = 1
            return
        else:            
            filename = self.filename = \
                       self.parent.a.submissions[self.username].pdf

            self.pages = self.parent.a.submissions[self.username].numPages

            self.checkBounds()
            self.bitmap = wx.Bitmap(self.makeFilename(self.username,
                                                     self.curpage))
            
        # Update title
        self.SetTitle("Annotate: " + self.username)
        self.subTextCtrl.SetValue(self.username)        

        self.redraw()

    def makeFilename(self, username, page):
        dirname = os.path.dirname(self.parent.a.submissions[username].pdf)
        return dirname + "/" + username + "-" + str(page) + ".ppm"

    def onNewComment(self, event):
        self.commentTree.onNewCommentContext(event)

    def checkBounds(self):
        # Check to see . . .
        # Are we the first/last submission?
        usernames = self.usernames
        if self.username == usernames[0]:
            self.prevSubBtn.Enable(False)
        else:
            self.prevSubBtn.Enable(True)

        if self.username == usernames[-1]:
            self.nextSubBtn.Enable(False)
        else:
            self.nextSubBtn.Enable(True)        
        

        # Are we out of bounds on pages for this submission (update page)
        if self.curpage > self.pages:
            self.curpage = self.pages
            self.pageTextCtrl.SetValue(str(self.curpage))

        if self.curpage == 1:            
            self.prevPageBtn.Enable(False)
        else:
            self.prevPageBtn.Enable(True)

        if self.curpage == self.pages:
            self.nextPageBtn.Enable(False)
        else:
            self.nextPageBtn.Enable(True)
        


    def onPrevPage(self, event):
        self.curpage -= 1
        self.updatePage()

    def onPrevSub(self, event):
        self.username = self.usernames[self.usernames.index(self.username) - 1]
        self.updateSub()

    def onNextSub(self, event):
        self.username = self.usernames[self.usernames.index(self.username) + 1]
        self.updateSub()
        
    def onNextPage(self, event):
        self.curpage += 1
        self.updatePage()

    def onDone(self, event=None):
        self.parent.subTree.update()
        self.parent.resultTree.update()
        
        self.Show(False)
        self.Destroy()
        self.parent.aFrame = None
        
        
# end of class AnnotateFrame


