import wx

import comments

class CommentDialog(wx.Dialog):
    def __init__(self, topNames, commentNames, comment, new, name, parent,
                 *args, **kwds):
        self.topNames = topNames
        self.commentNames = commentNames
        self.comment = comment
        self.new = new
        self.startingName = name
        self.name = name

        kwds["style"] = wx.CAPTION|wx.STAY_ON_TOP
        wx.Dialog.__init__(self, parent, *args, **kwds)
        self.nameLabel = wx.StaticText(self, -1, "Comment Name  ")
        self.nameTextCtrl = wx.TextCtrl(self, -1, self.startingName)
        self.rubricLabel = wx.StaticText(self, -1, "Rubric Item  ")
        self.rubricChoice = wx.Choice(self, -1, choices=topNames)
        self.pointLabel = wx.StaticText(self, -1, "Point Value  ")
        self.pointBox = wx.TextCtrl(self, -1, "")
        self.descriptionLabel = wx.StaticText(self, -1, "Description  ")
        self.description = wx.TextCtrl(self, -1, "", style=wx.TE_MULTILINE)
        self.panel_18 = wx.Panel(self, -1)
        self.addCurBox = wx.CheckBox(self, -1, "Add to current")
        self.appliesTo = wx.CheckBox(self, -1, "All rubric items")
        self.commentType = wx.RadioBox(self, -1, "Value Type",
                                      choices=["Add/subtract", "Percentage",
                                               "Set value"],
                                      majorDimension=1,
                                      style=wx.RA_SPECIFY_ROWS)
        self.commentOkBtn = wx.Button(self, -1, "OK")
        self.commentCancelButton = wx.Button(self, -1, "Cancel")

        self.__set_properties()
        self.__do_layout()

        if self.new:
            self.SetTitle("New Comment")
        else:
            self.SetTitle("Edit Comment")

        self.pointBox.SetValue(str(self.comment.value))
        self.description.SetValue(self.comment.comment)
        self.nameTextCtrl.SetValue(self.startingName)
            
        assert(self.comment.test.name in self.topNames)

        selInd = self.topNames.index(self.comment.test.name)
        self.rubricChoice.SetSelection(selInd)
        
        if not self.new:
            self.addCurBox.SetValue(False)
            self.addCurBox.Disable()

        if self.comment.appliesAll:
            self.appliesTo.SetValue(True)
            self.rubricChoice.Enable(False)

        if self.comment.op == "+":
            self.commentType.SetSelection(0)
        elif self.comment.op == "*":
            self.commentType.SetSelection(1)
        elif self.comment.op == "=":
            self.commentType.SetSelection(2)
                
        wx.EVT_BUTTON(self.commentOkBtn, -1, self.onOK)
        wx.EVT_BUTTON(self.commentCancelButton, -1, self.onCancel)
        wx.EVT_CHECKBOX(self.appliesTo, -1, self.onCheck)

    def __set_properties(self):
        self.SetTitle("New Comment")
        self.rubricChoice.SetSelection(0)
        self.description.SetSize((300, 104))
        self.addCurBox.SetToolTipString('If checked, will add this comment ' +\
                                        'to the current submission when you '+\
                                        'hit "OK"')
        self.addCurBox.SetValue(1)
        self.appliesTo.SetToolTipString('Click this to apply the effect of '+\
                                        'this comment to ALL test results.')
        self.commentType.SetSelection(0)
        self.commentOkBtn.SetDefault()

    def __do_layout(self):
        grid_sizer_13 = wx.FlexGridSizer(7, 2, 5, 5)
        grid_sizer_13.Add(self.nameLabel, 0,
                          wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_13.Add(self.nameTextCtrl, 1, wx.EXPAND, 0)
        grid_sizer_13.Add(self.rubricLabel, 0,
                          wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_13.Add(self.rubricChoice, 1, wx.EXPAND, 0)
        grid_sizer_13.Add(self.pointLabel, 0,
                          wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_13.Add(self.pointBox, 1, wx.EXPAND, 0)
        grid_sizer_13.Add(self.descriptionLabel, 0,
                          wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL, 0)
        grid_sizer_13.Add(self.description, 1,
                          wx.ADJUST_MINSIZE|wx.FIXED_MINSIZE, 0)
        grid_sizer_13.Add(self.panel_18, 1, wx.EXPAND, 0)
        grid_sizer_13.Add(self.addCurBox, 0,
                          wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL,
                          0)
        grid_sizer_13.Add(self.appliesTo, 0, 0, 0)
        grid_sizer_13.Add(self.commentType, 0, 0, 0)
        grid_sizer_13.Add(self.commentOkBtn, 0,
                          wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL,
                          0)
        grid_sizer_13.Add(self.commentCancelButton, 0,
                          wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL,
                          0)
        self.SetAutoLayout(1)
        self.SetSizer(grid_sizer_13)
        grid_sizer_13.Fit(self)
        grid_sizer_13.SetSizeHints(self)
        self.Layout()

    def onCheck(self, event):
        if self.appliesTo.IsChecked():
            self.otherwiseSelected = self.rubricChoice.GetSelection()
            self.rubricChoice.SetSelection(self.rubricChoice.GetCount() - 1)
            self.rubricChoice.Enable(False)
        else:
            self.rubricChoice.SetSelection(self.otherwiseSelected)
            self.rubricChoice.Enable(True)

    def onOK(self, event):       
        self.name = self.nameTextCtrl.GetValue()
        error = None
        if self.name in self.commentNames and \
               self.name != self.startingName:
            error = "Duplicate comment name"
        if not self.name:
            error = "You must specify a name"
        if error:
            dlg = wx.MessageDialog(self, error, "Error",
                                  wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return

        try:
            self.comment.value = float(self.pointBox.GetValue())
            assert(self.comment.value == round(self.comment.value, 1))
        except:
            dlg = wx.MessageDialog(self, "Points must be a number with no " +
                                  "more than 1 decimal place", "Error",
                                  wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return
    
        self.comment.comment = self.description.GetValue()        

        if self.appliesTo.IsChecked():
            self.comment.appliesTo = True
        else:
            self.comment.appliesTo = False

        if self.commentType.GetSelection() == 0:
            self.comment.commentType = "+"
        elif self.commentType.GetSelection() == 1:
            self.comment.commentType = "*"
        elif self.commentType.GetSelection() == 2:
            self.comment.commentType = "="

        self.EndModal(wx.ID_OK)

        
    def onCancel(self, event):
        self.EndModal(wx.ID_CANCEL)





