import wx

class TestConfigDialog(wx.Dialog):
    def __init__(self, parent, rubricTest, *args, **kwds):
        self.parent = parent
        self.rubricTest = rubricTest

        kwds["title"] = "Test Configuration"
        
        wx.Dialog.__init__(self, parent, *args, **kwds)

        self.okButton = wx.Button(self, -1, "OK")
        self.cancelButton = wx.Button(self, -1, "Cancel")

        flags = rubricTest.test.getFlagParams()
        files = rubricTest.test.getFileParams()
        vals =  rubricTest.test.getValParams()

        varWidth = 0
        if flags:
            varWidth += 1
        if files:
            varWidth += 3
        if vals:
            varWidth += 2

        if varWidth > 2:
            width = varWidth
        else:
            width = 2

        # Correct filename?

        height = 4
        if flags or files or vals:
            height += 1
        height += max([len(x) for x in flags, files, vals])

        grid = []
        for i in range(height):
            grid.append([None]*width)

        curCol = 0
        self.valMapping = {}
        if flags:
            grid[0][curCol] = wx.StaticText(self, -1, "Flags")
            for ind, val in enumerate(flags):
                grid[ind + 1][curCol] = wx.CheckBox(self, -1, val)
                self.valMapping[val] = grid[ind + 1][curCol]
            curCol += 1
            
        if vals:
            grid[0][curCol] = wx.StaticText(self, -1, "Parameters")
            for ind, val in enumerate(vals):
                configVal = rubricTest.config.get(val, "")
                grid[ind + 1][curCol] = wx.StaticText(self, -1, val)
                grid[ind + 1][curCol + 1] = wx.TextCtrl(self, -1,
                                                        configVal,
                                                        size=(175, 20))
                self.valMapping[val] = grid[ind + 1][curCol + 1]
            curCol += 2

        if files:
            self.buttonMapping = {}
            grid[0][curCol] = wx.StaticText(self, -1, "Files")
            for ind, val in enumerate(files):
                configVal = rubricTest.config.get(val, "")
                grid[ind + 1][curCol] = wx.StaticText(self, -1, val)
                grid[ind + 1][curCol + 1] = wx.TextCtrl(self, -1,
                                                        configVal,
                                                        size=(175, 20))
                grid[ind + 1][curCol + 2] = wx.Button(self, -1, "Select File")
                wx.EVT_BUTTON(grid[ind + 1][curCol + 2], -1, self.onFileButton)
                self.buttonMapping[grid[ind + 1][curCol + 2]] = val
                self.valMapping[val] = grid[ind + 1][curCol + 1]


        # Common stuff
        grid[len(grid) - 4][0] = wx.Button(self, -1, "Configure Autocomments")
        wx.EVT_BUTTON(grid[len(grid) - 4][0], -1, self.onAutoComments)
        
        grid[len(grid) - 3][0] = wx.StaticText(self, -1, "Filemask")
        self.fileMask = wx.TextCtrl(self, -1, rubricTest.getFileMask(),
                                    size=(175, 20))
        grid[len(grid) - 3][1] = self.fileMask

        grid[len(grid) - 2][0] = wx.StaticText(self, -1, "Points")
        self.points = wx.TextCtrl(self, -1, str(rubricTest.getPoints()))
        grid[len(grid) - 2][1] = self.points

        grid[len(grid) - 1][-1] = self.cancelButton
        grid[len(grid) - 1][-2] = self.okButton

        self.sizer = wx.FlexGridSizer(height, width, 5, 5)
        for row in grid:
            for val in row:
                if val == None:
                    self.sizer.Add(wx.Panel(self))
                    continue
                
                if isinstance(val, wx.StaticText):
                    self.sizer.Add(val, 0, wx.ALIGN_RIGHT|wx.ALL, 3)
                elif isinstance(val, wx.Button):
                    self.sizer.Add(val, 1, wx.ALIGN_CENTER|wx.ALL, 3)
                else:
                    self.sizer.Add(val, 1, wx.EXPAND|wx.ALL, 3)
        

        self.SetSizer(self.sizer)
        self.sizer.SetSizeHints(self)

        wx.EVT_BUTTON(self.okButton, -1, self.onOK)
        wx.EVT_BUTTON(self.cancelButton, -1, self.onCancel)

        wx.EVT_SIZE(self, self.onSize)

    def onSize(self, event):
        self.sizer.SetSizeHints(self)        
        self.Layout()

    def onOK(self, event):
        try:
            self.rubricTest.setPoints(float(self.points.GetValue()))
        except ValueError:
            dlg = wx.MessageDialog(self,
                                   "Point value must be numeric",
                                   "Error",
                                   style=wx.ICON_ERROR|wx.OK)
            dlg.ShowModal()
            dlg.Destroy()
            return

        self.rubricTest.setFileMask(str(self.fileMask.GetValue()))
        
        for val, widget in self.valMapping.items():
            if not isinstance(widget, wx.CheckBox):
                self.rubricTest.config[val] = str(widget.GetValue())
            else:
                self.rubricTest.config[val] = widget.GetValue()            

        self.EndModal(wx.ID_OK)

    def onCancel(self, event):
        self.EndModal(wx.ID_CANCEL)

    def onFileButton(self, event):
        val = self.buttonMapping[event.GetEventObject()]

        dlg = wx.FileDialog(self, style=wx.OPEN)
        ret = dlg.ShowModal()
        path = dlg.GetPath()
        dlg.Destroy()

        if ret == wx.ID_CANCEL:
            return

        self.valMapping[val].SetValue(path)
    
    def onAutoComments(self, event):
        # TODO: Deal with this
        print "Auto comments"
