import sys
import gzip
import re
import string
import tempfile
import commands
import os

from xml.sax import *

VAL_TYPE_FUNC = 0
VAL_TYPE_INT = 30
VAL_TYPE_FLOAT = 40
VAL_TYPE_STR = 60

class GnumeriPy(handler.ContentHandler):
    def __init__ (self, filename):
        self.sheets = []
        self.sheetNameLookup = {}
        self.selected = None

        self.recent = []

        if filename.endswith(".xls"):
            self.origName = filename
            filename = filename.replace(".xls", ".gnumeric")
            commands.getoutput("ssconvert %s %s" % (self.origName, filename))
        
        self.filename = filename
        self.xml = gzip.open(filename).readlines()

        parseString("".join(self.xml), self)

        self.xml = [x.rstrip() for x in self.xml]
        
    def __getitem__(self, key):
        if type(key) == type(0):
            return self.sheets[key]
        else:
            return self.sheets[self.sheetNameLookup[unicode(key)]]

    def startElement(self, name, attrs):
        self.recent.append(name)

        if name == "gnm:Cell":
            self.col = int(attrs["Col"])
            self.row = int(attrs["Row"])
            
            try:
                self.valType = int(attrs["ValueType"])
            except:
                self.valType = VAL_TYPE_FUNC
                
            try:
                self.exprID = int(attrs["ExprID"])
            except:
                self.exprID = None
            
            self.charData = ""

        if name == "gnm:UIData":
            self.selected = int(attrs["SelectedTab"])

    
    def endElement(self, name):
        if self.recent[-1] == "gnm:Cell":
            self.sheets[-1].insert(self.row,
                                   self.col,
                                   self.valType,
                                   self.exprID,
                                   self.charData)
        self.recent.pop()


    def characters(self, content):        
        if self.recent[-2:] == ["gnm:Sheet", "gnm:Name"]:
            self.sheetNameLookup[content] = len(self.sheets) 
            self.sheetNameLookup[len(self.sheets)] = content
            self.sheets.append(Sheet(content, self))


        if self.recent[-1] == "gnm:Cell":
            self.charData += content

    def mkTag(self, name, attrs):
        ret = "<%s" % name
        for key in attrs.keys():
            ret += ' %s="%s"' % (key, attrs[key])
        ret += " />"
        return ret

    def addSheet(self, name):
        self.sheetNameLookup[len(self.sheets)] = name
        self.sheetNameLookup[name] = len(self.sheets)
        self.sheets.append(Sheet(name, self))

        # Add to the sheet name index
        for ind, val in enumerate(self.xml):
            if val.strip() == "</gnm:SheetNameIndex>":
                self.xml[ind:ind] = ["<gnm:SheetName>%s</gnm:SheetName>" %name]
                break

        # Add a blank for the sheet to the end of Sheets
        for ind, val in enumerate(self.xml):
            if val.strip() == "</gnm:Sheets>":
                self.xml[ind:ind] = newSheet(name)
                break

    def sheetNames(self):
        return [str(x) for x in self.sheetNameLookup.keys() \
                if type(x) in [type(""), type(u"")]]

    def selectSheet(self, toSelect):
        if type(toSelect) in (type(""), type(u"")):
            toSelect = self.sheetNameLookup[toSelect]
        assert(type(toSelect) == type(0) and 0 <= toSelect < len(self.sheets))
        self.selected = toSelect

    def cleanExpRefs(self, row, col, formula, expid):
        for sheet in self.sheets:
            sheet.cleanExpRefs(row, col, formula, expid)                    

    def freeze(self):
        for sheet in self.sheets:
            sheet.freeze()

    def saveAs(self, filename):
        origName = None
        if filename.endswith(".xls"):
            origName = filename
            filename = origName.replace(".xls", ".gnumeric")            
        
        of = gzip.open(filename, 'w')

        curSheet = None
        printed = False
        for line in self.xml:
            if line.strip().startswith("<gnm:Name>"):
                sInd = line.index(">") + 1
                eInd = line.index("<", sInd)
                curSheet = line[sInd:eInd]
                printed = False
            if line.strip().startswith("<gnm:UIData "):
                print >>of, '<gnm:UIData SelectedTab="%d"/>' % self.selected
            elif line.strip().startswith("<gnm:MaxCol"):
                cols = self.sheets[self.sheetNameLookup[curSheet]].cols()
                print >>of, "<gnm:MaxCol>%d</gnm:MaxCol>" % cols
            elif line.strip().startswith("<gnm:MaxRow"):
                rows = self.sheets[self.sheetNameLookup[curSheet]].rows()
                print >>of, "<gnm:MaxRow>%d</gnm:MaxRow>" % rows            
            elif not line.strip().startswith("<gnm:Cell "):
                print >>of, line
                continue
            else:
                if not printed:
                    printed = True
                    sheet = self.sheets[self.sheetNameLookup[curSheet]]
                    for line in sheet.toXML():
                        print >>of, line

        of.close()

        if origName:
            commands.getstatusoutput("ssconvert %s %s" % (filename, origName))

class Sheet:
    def __init__(self, name, parent):
        self.name = name
        self.parent = parent

        self.data = []

        self.indexRow = 0
        self.indexCol = 0

    def freeze(self):
        oldSelected = self.parent.selected
        self.parent.selectSheet(self.name)
        (fh, fn) = tempfile.mkstemp(".gnumeric")
        (fhCSV, tempCSV) = tempfile.mkstemp(".csv")
        self.parent.saveAs(fn)
        self.parent.selectSheet(oldSelected)
        (s, o) = commands.getstatusoutput("ssconvert %s %s" % (fn, tempCSV))

        if s:
            raise RuntimeError("Error converting to csv with ssconvert")

        os.close(fh)
        os.close(fhCSV)
        os.unlink(fn)

        self.data = [self.parseCSLine(line) for line in open(tempCSV)]
        os.unlink(tempCSV)
        
    def parseCSLine(self, line):
        ret = []
        prev = 0
        inQuotes = False
        for ind in range(len(line)):
            if line[ind] == '"':
                inQuotes = not inQuotes
            if line[ind] == ',' and not inQuotes:
                ret.append(line[prev:ind])                    
                prev = ind + 1
        if line[-1] != ",":
            ret.append(line[prev:])
        
        for ind, r in enumerate(ret):
            if len(r) and r[0] == r[-1] == '"':
                ret[ind] = r[1:-1]
            try:
                ret[ind] = int(ret[ind])
            except:
                try:
                    ret[ind] = float(ret[ind])
                except:
                    pass

        return ret
            
    def saveCSV(self, filename, convert=False):
        if not convert:
            of = open(filename, 'w')
            print >>of, str(self)
            of.close()
        else:
            oldSelected = self.parent.selected
            self.parent.selectSheet(self.name)
            (fh, fn) = tempfile.mkstemp(".gnumeric")
            self.parent.saveAs(fn)
            self.parent.selectSheet(oldSelected)
            (s, o) = commands.getstatusoutput("ssconvert %s %s" % \
                                              (fn, filename))
            if s:
                raise RuntimeError("Error converting to csv with ssconvert")

            os.close(fh)
            os.unlink(fn)
            
    def setIndexRow(self, row):
        assert(type(row) == type(0) and 0 <= row < len(self.data))
        self.indexRow = row

    def setIndexCol(self, col):
        assert(type(col) == type(0) and 0 <= col < self.cols())
        self.indexCol = col

    def getRow(self, key):
        # Find the row st row[indexCol] == key and return it
        for row in self.data:
            if row[self.indexCol] == key:
                return row

        raise KeyError

    def getCol(self, key):
        return [row[self.indexCol] for row in self.data]

    def getKeyRow(self, key):
        key = unicode(key)
        for ind, row in enumerate(self.data):
            if len(row) > self.indexCol and row[self.indexCol] == key:
                return ind
        raise ValueErorr

    def getKeyCol(self, key):
        key = unicode(key)
        for ind, val in enumerate(self.data[self.indexRow]):
            if val == key:
                return ind
        raise ValueError

    def hasKeyRow(self, key):
        for row in self.data:
            if len(row) > self.indexCol and row[self.indexCol] == key:
                return True
        return False

    def hasKeyCol(self, key):
        for val in self.data[self.indexRow]:
            if val == key:
                return True
        return False

    def expand(self, row, col):
        if row >= len(self.data):
            self.data += [[] for x in range(row - len(self.data) + 1)]

        if col >= len(self.data[row]):
            self.data[row] += [None for x in range(col-len(self.data[row])+ 1)]

    def set(self, row, col, val):
        self.expand(row, col)

        # If this is the master entry for some expRef . . . 
        if type(self.data[row][col]) == type(()) and \
           None not in self.data[row][col]:
            self.parent.cleanExpRefs(row, col,
                                     self.data[row][col][0],
                                     self.data[row][col][1])
                        
        self.data[row][col] = val

    def cleanExpRefs(self, row, col, formula, expid):
        # Look through everything else to find the expRefs we
        for r, rowData in enumerate(self.data):
            for c, v in enumerate(rowData):
                # This one needs to be converted
                if type(v) == type(()) and \
                   v == ('', expid):
                    self.data[r][c] = convertExpRef(r, c, v[0], v[1],
                                                    formula,
                                                    row, col)        

    def get(self, row, col):
        return self.data[row][col]

    def hasData(self, row, col):
        try:
            if self.data[row][col] == None:
                return False
            return True
        except:
            return False

    def rows(self):
        return len(self.data)

    def cols(self):
        return max([len(x) for x in self.data])

    def insert(self, row, col, valType, expid, data):
        self.expand(row, col)
        
        if valType == VAL_TYPE_STR:
            self.data[row][col] = data
        elif valType == VAL_TYPE_INT:
            self.data[row][col] = int(data)
        elif valType == VAL_TYPE_FLOAT:
            self.data[row][col] = float(data)
        elif valType == VAL_TYPE_FUNC:
            self.data[row][col] = (data, expid)
        else:
            assert(expid != None and valType == None)
            self.data[row][col] = None

    def toXML(self):
        ret = []
        for r, row in enumerate(self.data):
            for c, val in enumerate(row):
                if val == None:
                    continue
                if type(val) in [type(""), type(u"")]:
                    cur = '<gnm:Cell Col="%d" Row="%d" ValueType="%d">%s</gnm:Cell>' % (c, r, VAL_TYPE_STR, val)
                if type(val) == type(0):
                    cur = '<gnm:Cell Col="%d" Row="%d" ValueType="%d">%d</gnm:Cell>' % (c, r, VAL_TYPE_INT, val)
                if type(val) == type(0.0):
                    cur = '<gnm:Cell Col="%d" Row="%d" ValueType="%d">%f</gnm:Cell>' % (c, r, VAL_TYPE_FLOAT, val)
                if type(val) == type(()):
                    if val[1] == None:
                        cur = '<gnm:Cell Col="%d" Row="%d">%s</gnm:Cell>' % (c, r, val[0])
                    else:
                        if val[0] != None:
                            cur = '<gnm:Cell Col="%d" Row="%d" ExprID="%d">%s</gnm:Cell>' % (c, r, val[1], val[0])
                        else:
                            cur = '<gnm:Cell Col="%d" Row="%d" ExprID="%d" />' % (c, r, val[1])
                            
                ret.append(cur)

        return ret                        

    def __str__(self):
        ret = []
        for row in self.data:
            s = ""
            for val in row:
                if type(val) == type(()):
                    val = None

                if val == None:
                    s += ","
                elif type(val) in [type(""), type(u"")]:
                    if "," in val or " " in val:
                        s += '"' + val + '",'
                    else:
                        s += val + ","
                else:
                    s += str(val) + ","
            if len(row) < self.cols():
                s += "," * (self.cols() - len(row))
                    
            ret.append(s[:-1])
        return "\n".join(ret)

def newSheet(name):
    data = newSheetData % name
    return data.split("\n")

def coordFromName(cell):
    if "!" in cell:
        m = re.match("([A-Za-z0-9_']+)!([A-Za-z]+)([0-9]+)", cell)
        assert(m != None)
        (sheet, col, row) = m.groups()
    else:
        m = re.match("([A-Za-z]+)([0-9]+)", cell)
        assert(m != None)
        sheet = ""
        col, row = m.groups()

    row = int(row)
    row -= 1
    col = string.upper(col)
    c = 0
    while col:
        c *= 26
        c += ord(col[0]) - ord('A') + 1
        col = col[1:]
    c -= 1
    return sheet, c, row

def cellName(row, col):
    """Return the Gnumeric name of this cell, given its 0-based row/col"""
    # TODO: This doesn't work beyond column YZ    
    if col < 0:
        col = 0
    row += 1
    mod = col % 26
    div = col / 26
    ret = chr(ord("A") + mod)
    tweak = 1
    while div:
        col /= 26
        mod = col % 26
        div = col / 26
        ret = chr(ord("A") + mod - tweak) + ret
        tweak = 0
    return ret + str(row)

def convertExpRef(row, col, line, eid, formula, frow, fcol):
    # Pull out anything quoted (like names of sheets)
    reducedFormula = formula
    goods = []

    while reducedFormula.count("'") >= 2:
        start = reducedFormula.index("'")
        end = reducedFormula.index("'", start + 1)
        assert(start != end and
               reducedFormula[start] == reducedFormula[end] == "'")

        if reducedFormula[end + 1] == "!":
            end += 2
            while reducedFormula[end] in string.uppercase:
                end += 1            
            while end < len(reducedFormula) and \
                  reducedFormula[end] in string.digits:                      
                end += 1

            goods.append(reducedFormula[start:end])
            reducedFormula = reducedFormula[:start] + reducedFormula[end + 1:]
            
        reducedFormula = reducedFormula[:start] + "tempsheet" + \
                         reducedFormula[end + 1:]
        
    pattern = re.compile("((?:\w+\!)?[A-Za-z]+[0-9]+)")
    possibles = pattern.split(reducedFormula)

    for pos in possibles:
        if pattern.split(pos) != [pos] and pos not in goods:
            goods.append(pos)

    ##########
    # Now for each element in goods, convert it relative to the new
    # row and column

    # First figure out where we are relative to the formula declaration
    dcol = col - fcol
    drow = row - frow

    cellMap = {}
    for cell in goods:
        # And then figure out where we were
        (sheet, ecol, erow) = coordFromName(cell)
        
        if sheet:
            newCell = sheet + "!" + cellName(erow + drow, ecol + dcol)
        else:
            newCell = cellName(erow + drow, ecol + dcol)
    
        # Now for each time the old formulas occur, replace them with the
        # new and improved version
        newFormula = formula.replace(cell, newCell)         
        formula = newFormula

    return (formula, None)

newSheetData = """    <gnm:Sheet DisplayFormulas="0" HideZero="0" HideGrid="0" HideColHeader="0" HideRowHeader="0" DisplayOutlines="1" OutlineSymbolsBelow="1" OutlineSymbolsRight="1" Visibility="GNM_SHEET_VISIBILITY_VISIBLE">
      <gnm:Name>%s</gnm:Name>
      <gnm:MaxCol>0</gnm:MaxCol>
      <gnm:MaxRow>0</gnm:MaxRow>
      <gnm:Zoom>1</gnm:Zoom>
      <gnm:PrintInformation>
        <gnm:Margins>
          <gnm:top Points="120" PrefUnit="cm"/>
          <gnm:bottom Points="120" PrefUnit="cm"/>
        </gnm:Margins>
        <gnm:Scale type="percentage" percentage="100"/>
        <gnm:vcenter value="0"/>
        <gnm:hcenter value="0"/>
        <gnm:grid value="0"/>
        <gnm:even_if_only_styles value="0"/>
        <gnm:monochrome value="0"/>
        <gnm:draft value="0"/>
        <gnm:titles value="0"/>
        <gnm:order>d_then_r</gnm:order>
        <gnm:orientation>portrait</gnm:orientation>
        <gnm:Header Left="" Middle="&amp;[TAB]" Right=""/>
        <gnm:Footer Left="" Middle="Page &amp;[PAGE]" Right=""/>
      </gnm:PrintInformation>
      <gnm:Styles>
        <gnm:StyleRegion startCol="0" startRow="0" endCol="255" endRow="65535">
          <gnm:Style HAlign="1" VAlign="2" WrapText="0" ShrinkToFit="0" Rotation="0" Shade="0" Indent="0" Locked="1" Hidden="0" Fore="0:0:0" Back="FFFF:FFFF:FFFF" PatternColor="0:0:0" Format="General">
            <gnm:Font Unit="10" Bold="0" Italic="0" Underline="0" StrikeThrough="0">Sans</gnm:Font>
            <gnm:StyleBorder>
              <gnm:Top Style="0"/>
              <gnm:Bottom Style="0"/>
              <gnm:Left Style="0"/>
              <gnm:Right Style="0"/>
              <gnm:Diagonal Style="0"/>
              <gnm:Rev-Diagonal Style="0"/>
            </gnm:StyleBorder>
          </gnm:Style>
        </gnm:StyleRegion>
      </gnm:Styles>
      <gnm:Cols DefaultSizePts="48">
        <gnm:ColInfo No="0" Unit="48" MarginA="2" MarginB="2" Count="3"/>
      </gnm:Cols>
      <gnm:Rows DefaultSizePts="12.75">
        <gnm:RowInfo No="0" Unit="12.75" MarginA="0" MarginB="0"/>
      </gnm:Rows>
      <gnm:Selections CursorCol="2" CursorRow="1">
        <gnm:Selection startCol="2" startRow="1" endCol="2" endRow="1"/>
      </gnm:Selections>
      <gnm:Cells>
        <gnm:Cell Col="0" Row="0" ValueType="30">1</gnm:Cell>
        <gnm:Cell Col="1" Row="0" ValueType="30">2</gnm:Cell>
        <gnm:Cell Col="2" Row="0" ExprID="1"/>
      </gnm:Cells>
      <gnm:SheetLayout TopLeft="A1"/>
      <gnm:Solver ProblemType="1" Inputs="" MaxTime="0" MaxIter="0" NonNeg="1" Discr="0" AutoScale="0" ShowIter="0" AnswerR="0" SensitivityR="0" LimitsR="0" PerformR="0" ProgramR="0"/>
    </gnm:Sheet>"""

if __name__ == "__main__":
    g = GnumeriPy(sys.argv[1])


    s = g.sheets[0]
    g.selectSheet(1)

    s.set(1, 6, "Haha!")

    g.saveAs("test.gnumeric")

    g.freeze()
    g.saveAs("frozen.gnumeric")

    g.addSheet("newSheet")
    s = g["newSheet"]
    s.set(3, 4, "Hello World!")
    print g.sheetNames()
    g.saveAs("new.gnumeric")
