import wx

from memTree import *
from droptarget import *

class ResultTree(MemTree):
    """This memtree based widget displayes the results for each student."""
    def __init__(self, parent, agar, *args, **kwds):
        MemTree.__init__(self, "Results", *args, **kwds)

        self.a = agar
        self.parent = parent
        self.subtree = self.parent.subTree        

        self.dt = AgarDropTarget(self.getDropData)
        self.data = wx.CustomDataObject(wx.CustomDataFormat("AgarComment"))
        self.dt.SetDataObject(self.data)
        self.SetDropTarget(self.dt)
        
        # Do the first draw
        self.update()        
        
        wx.EVT_TREE_SEL_CHANGED(self.subtree, -1, self.onSubChanged)
        wx.EVT_TREE_SEL_CHANGED(self, -1, self.onChanged)

    def getDropData(self, x, y, data):
        """Allow drag and drop of comments"""
        if self.dt.GetData():
            commentName = str(self.data.GetData())

            if commentName not in self.a.userComments:
                return

            try:
                sub = self.a.submissions[self.subtree.getUsername()]
            except:
                return

            sub.comments.addComment(self.a.userComments[commentName])

            self.subtree.update()

    def onSubChanged(self, event):
        """Make sure things are drawn correctly"""
        self.subtree.onSelected(event)
        self.update()

    def onChanged(self, event):
        """Make sure things are drawn correctly"""
        MemTree.onSelected(self, event)
        self.parent.resultWin.update()

    def reset(self):
        """Just redraw completely, although this resets all
        expanded-node information"""
        self.tree.children = []
        self.update()

    def update(self):
        """Draw the tree: usernames have a list of top-level rubric
        items, and then auto-comments and user-comments that have been
        assigned to each of those."""
        try:
            # Get the submission
            sub = self.a.submissions[self.subtree.getUsername()]
        except AssertionError:
            return

        self.unTag()

        self.tree.name = sub.getUsername()

        # Regenerate tests
        tests = [(x, x.name) for x in self.a.rubric.getTopLevel()]

        for ind, (test, name) in enumerate(tests):
            toolName = "%s: %s" % \
                          (name, str(sub.getScore(self.a.rubric[name])))
            self.pathAdd([toolName], ind=ind)

            # Add the comments
            self.pathAdd([toolName, "Auto Comments"], True)
            self.pathAdd([toolName, "User Comments"], True)

            # Only insert comments for the things for this test.
            for c in sub.comments:
                if not test.hasChild(c.test):
                    continue

                if c.appliesAll:
                    self.pathAdd(["Applies To All",
                                  "(%s%s)" % (c.op, str(c.value))],
                                 data=c)
                    continue
                    
                if c.auto:
                    self.pathAdd([toolName, "Auto Comments",
                                  c.test.name + " (%s%s)" %\
                                  (c.op, str(c.value))],
                                 data=c)

                else:
                    cName = None
                    for uc in self.a.userComments:
                        if self.a.userComments[uc] is c:
                            cName = uc
                            break

                    assert(cName is not None)
                    
                    self.pathAdd([toolName, "User Comments",
                                  cName + " (%s%s)" % (c.op, str(c.value))],
                                 data=c)


        # Add the final score
        self.pathAdd(["Total: %s" % str(sub.getScore())])

        self.prune()
        MemTree.update(self)
        

        
