#!/usr/bin/python

import commands
import getopt
import os
import os.path
import popen2
import select
import signal
import string
import sys
import tempfile
import time
import thread


def usage():
    print "Usage: difftest.py [OPTION]... EXECUTABLE points"
    print "Does the output of this executable match the expected output?"
    print
    print "--infile=FILE \t\tWhat input should be given to the executable."
    print "--outfile=FILE\t\tWhat output the executable must produce on "
    print "              \t\tthat input."
    print "--filter=FILE \t\tAn executable that reads stdin, filters it"
    print "              \t\thowever you like, and writes to stdout."
    print "--timeout=secs\t\tHow many seconds before giving up."
    print "--as-argument \t\tGive the program the argument 'infile' rather"
    print "              \t\tthan sending the contents of infile to standard"
    print "              \t\tinput."
    print "--flags-to-diff=FLAGS\t\tOptional flags to be passed to diff.  See"
    print "               \t\tthe man page for diff for more info."
    
    sys.exit(2)

def defaults():
    print "Diff Test"
    print "one"
    print "*graderExe"
    print "{'timeout': 3}"
    usage()

try:
    opts, args = getopt.getopt(sys.argv[1:], "", ["help", "defaults",
                                                  "infile=",
                                                  "outfile=",
                                                  "filter=",
                                                  "timeout=",
                                                  "as-argument",
                                                  "flags-to-diff=" ])

    
except getopt.GetoptError:
    # print help information and exit:
    usage()
    sys.exit(1)

infile = None
outfile = None
iofilter = None
timeout = 3
asArg = False
diffFlags = ""
    
for o, a in opts:
    if o == "--help":
        usage()
    if o == "--defaults":
        defaults()

    try:        
        if o == "--infile":
            infile = a

        if o == "--outfile":
            outfile = a

        if o == "--filter":
            iofilter = a

        if o == "--timeout":
            timeout = float(a)

        if o == "--as-argument":
            asArg = True

        if o == "--flags-to-diff":
            diffFlags = a

    except:
        usage()

if not outfile:
    print "Missing output file"
    usage()

if infile:
    infile = os.path.expanduser(infile)
    
outfile = os.path.expanduser(outfile)

if iofilter:
    iofilter = os.path.expanduser(iofilter)

try:
    points = float(sys.argv.pop())
except:
    sys.exit(1)

try:
    file = open(sys.argv[-1])
except:
    print "Unable to open", sys.argv[-1], "for reading"
    usage()
    
raw = open(outfile).readlines()
data = ""
for i in raw:
    data += i
#data = data[:-1]

os.chdir(os.path.dirname(sys.argv[-1]))
sys.argv[-1] = "./" + os.path.basename(sys.argv[-1])

if infile:
    if asArg:
        childProc = popen2.Popen3(sys.argv[-1] + " " + infile)
    else:
        childProc = popen2.Popen3(sys.argv[-1] + "<" + infile)
else:
    childProc = popen2.Popen3(sys.argv[-1])

startTime = time.time()
output = ""
while 1:
    cur = time.time()
    timeLeft = timeout - (cur - startTime)

    if timeLeft < 0:
        break
    
    (r, w, o) = select.select([childProc.fromchild], [], [], timeLeft)

    if not r:
        timeLeft = 0
        break

    if childProc.fromchild.closed:
        break

    curline = childProc.fromchild.readline()
    if not curline:
        break

    output += curline

if timeLeft <= 0:
    print 0
    print "Timeout after", timeout, "seconds, aborting."    
    raw = commands.getoutput('ps -ef | grep "\\W' + str(childProc.pid) + \
                             '\\W"')
    lines = raw.split("\n")
    for line in lines:
        pid = int(line.split()[1])
        if pid != childProc.pid:
            os.kill(pid, signal.SIGKILL)
            
    os.kill(childProc.pid, signal.SIGINT)
    status = childProc.wait()
    if status:
        if status == 35584:
            print "Segmentation Fault"
        elif status == 34304:
            print "Aborted"
        else:
            print "Unexpected termination, aborting with exit status", status
    
    sys.exit(0)

if (output != data):
    (file, filename) = tempfile.mkstemp()
    os.write(file, output + "\n")

    cmdline = ""
    if iofilter:
        cmdline = "cat %s | %s | diff %s - %s" % (filename, iofilter,
                                                  diffFlags, outfile)
    else:
        cmdline = "diff %s %s %s" % (diffFlags, outfile, filename)
        
    (status, output) = commands.getstatusoutput(cmdline)
    if not status:
        print points
        print "No significant output differences found."
        sys.exit(0)

    print 0
    print cmdline
    print "diff " + diffFlags + " " + outfile + " " + filename
    print output
    sys.exit(0)

print points
print "No output differences found."
sys.exit(0)
