#!/usr/bin/python

import sys
import getopt
import os
import re


def usage():
    print "Usage: dosdetect.py FILE points"
    print "Check windows style line-feeds in the text of file FILE.  Fail if",
    print "detected."
    print
    print "    --invert\t\tSuccess if windows style line-feeds are found,"
    print "            \t\tfailure otherwise."
    sys.exit(2)

def defaults():
    print "Fail on Windows Code"
    print "all"
    print "*.c,*.cc,*.cpp,*.C,*.h"
    usage()


try:
    opts, args = getopt.getopt(sys.argv[1:], "", ["help", "defaults",
                                                  "invert"])
    
except getopt.GetoptError:
    # print help information and exit:
    usage()


matchRet = 1
for o, a in opts:
    if o == "--help":
        usage()
    if o == "--defaults":
        defaults()

    try:        
        if o == "--invert":
            matchRet = 0

    except:
        usage()

try:
    points = float(sys.argv[-1])
    sys.argv.pop()
except:
    sys.exit(1)

try:
    file = open(sys.argv[-1])
except:
    print "Unable to open", sys.argv[-1], "for reading"
    usage()

mainbad = 0
badlinecount = 0
linecount = 0
for line in file:
    linecount += 1
    if "\r\n" in line:
        badlinecount += 1
    if "void main" in line:
        mainbad = 1


if badlinecount or mainbad:
    if not matchRet:
        print points
    else:
        print 0
        
    if linecount:
        print "Windows-style lines: %d/%d (%.3f%%)" % (badlinecount,
                                                       linecount,
                                                       100.0 * badlinecount /
                                                       linecount)
    if mainbad:
        print "Main function declared as void"
        
    sys.exit(0)


if not matchRet:
    print 0
else:
    print points

print "No evidence of Windows code found."
sys.exit(0)



