#!/usr/bin/python

import sys
import getopt
import os
import string
import dircache
import datetime

def usage():
    print "Usage: pointlate.py [OPTIONS] FILE points"
    print "Returns a number of points (positive or negative) based"
    print "on submission turnin times.  REQUIRES UCR turnin-style"
    print "submission directories with full date information."
    print
    print "--3-late=POINTS\tHow many points for 3 days late? (Default -31)"
    print "--2-late=POINTS\tHow many points for 2 days late? (Default -21)"
    print "--1-late=POINTS \tHow many points for 1 day late? (Default -11)"
    print "--1-early=POINTS\tHow many points for 1 day early? (Default 2)"    
    print "--2-early=POINTS\tHow many points for 2 days early? (Default 4)"
    print "--3-early=POINTS\tHow many points for 3 days early? (Default 6)"
    print "--due-date=DATE\tWhen the actual due date/time was (HH:mm MM/DD/YY)"
    
    sys.exit(2)

def defaults():
    print "Early/Late Points"
    print "all"
    print "*"
    print "{'3 late': -31, '2 late': -21, '1 late': -11, '1 early': 2, '2 early': 4, '3 early': 6, 'due date': '20:00 02/28/06'}"
    usage()


try:
    opts, args = getopt.getopt(sys.argv[1:], "", ["help", "defaults",
                                                  "3-late=",
                                                  "2-late=",
                                                  "1-late=",
                                                  "1-early=",
                                                  "2-early=",
                                                  "3-early=",
                                                  "due-date="])

    
except getopt.GetoptError:
    # print help information and exit:
    print "Getopt error"
    usage()
    sys.exit(0)

lates = [-11.0, -21.0, -31.0]
earlies = [2.0, 4.0, 6.0]
date = None
for o, a in opts:
    if type(a) == type(""):
        a = string.strip(a)
        
    if o == "--help":
        usage()
    if o == "--defaults":
        defaults()
    try:
        if o == "--3-late":        
            lates[2] = round(float(a), 1)
        if o == "--2-late":
            lates[1] = round(float(a), 1)
        if o == "--1-late":
            lates[0] = round(float(a), 1)
        if o == "--1-early":
            earlies[0] = round(float(a), 1)
        if o == "--2-early":
            earlies[1] = round(float(a), 1)
        if o == "--3-early":
            earlies[2] = round(float(a), 1)
        if o == "--due-date":
            time, date = a.split(" ")
            hour, minute = time.split(":")
            month, day, year = date.split("/")
    except:
        raise
        usage()

            
if date == None:
    print "No date"
    usage()

try:
    points = float(args.pop())
except:
    print "Bad point value"
    sys.exit(2)

dirs = os.listdir(os.path.dirname(os.path.dirname(args[-1])))

if 'LATE' in dirs:
    del dirs[dirs.index("LATE")]
if 'ONTIME' in dirs:
    del dirs[dirs.index("ONTIME")]

dirs.sort()

#print dirs

last = dirs[-1]

username, asname, syear, smonth, sday, shour, smin, ssec = last.split(".")[:8]

target = datetime.datetime(int("20" + year), int(month), \
                           int(day), int(hour), int(minute))
student = datetime.datetime(int(syear), int(smonth), \
                            int(sday), int(shour), int(smin))

result = target - student

if result.days == 0:
    print 0
elif result.days == 1:
    print earlies[0]
    print "One day early, %.1f point bonus" % earlies[0]

elif result.days == 2:
    print earlies[1]
    print "Two days early, %.1f point bonus" % earlies[1]
    
elif result.days >= 3:
    print earlies[2]
    print "Three (or more) days early, %.1f point bonus" % earlies[2]
    
elif result.days == -1:
    print lates[0]
    print "One day late, %.1f point penalty" % abs(lates[0])
    
elif result.days == -2:
    print lates[1]
    print "Two days late, %.1f point penalty" % abs(lates[1])
    
elif result.days <= -3:
    print lates[2]
    print "Three (or more) days late, %.1f point penalty" % abs(lates[2])

sys.exit(0)
